local ldb_tip, lowest, total_price
local cost_tip = CreateFrame("GameTooltip")
local crayon   = LibStub("LibCrayon-3.0")
local frame    = CreateFrame("Button", "Attrition")
local ldb_obj  = LibStub("LibDataBroker-1.1"):NewDataObject("Attrition", {
	icon      = "Interface\\Icons\\Trade_BlackSmithing.png",
	label     = "Attrition"
})

AttritionDB = AttritionDB or { repair = false, lowest = false, guild = false }

local slots         = {}
slots.Head          = true
slots.Shoulder      = true
slots.Chest         = true
slots.Wrist         = true
slots.Hands         = true
slots.Waist         = true
slots.Legs          = true
slots.Feet          = true
slots.MainHand      = true
slots.SecondaryHand = true
slots.Ranged        = true

local function inform(text)
	DEFAULT_CHAT_FRAME:AddMessage("|cffffd700Attrition:|r |cffeeeeee"..text.."|r")
end

local function cash_to_string(cash)
	if not cash then return "?" end

	local gold   = floor(cash / (100 * 100))
	local silver = math.fmod(floor(cash / 100), 100)
	local copper = math.fmod(floor(cash), 100)
	gold         = gold   > 0 and "|cffeeeeee"..gold  .."|r|cffffd700g|r" or ""
	silver       = silver > 0 and "|cffeeeeee"..silver.."|r|cffc7c7cfs|r" or ""
	copper       = copper > 0 and "|cffeeeeee"..copper.."|r|cffeda55fc|r" or ""
	copper       = (silver ~= "" and copper ~= "") and " "..copper or copper
	silver       = (gold   ~= "" and silver ~= "") and " "..silver or silver

	return gold..silver..copper
end

local function set_ldb_obj_text(_, event, addon)
	if event == "ADDON_LOADED" and addon ~= "Attrition" then return end

	if event == "MERCHANT_SHOW" and AttritionDB.repair and CanMerchantRepair() then
		local cost = GetRepairAllCost()

		if cost > 0 then
			inform("Items repaired, the repairs cost "..cash_to_string(cost)..".")

			local guild = (AttritionDB.guild and IsInGuild() and CanGuildBankRepair()) and 1 or nil

			RepairAllItems(guild)
		end

		return
	end

	local average, count

	lowest, total_price = nil, nil

	for slot in pairs(slots) do
		local link = GetInventoryItemLink("player", slot.."Slot")

		if link then
			local id = GetInventorySlotInfo(slot.."Slot")

			local durability, max_durability

			durability, max_durability = GetInventoryItemDurability(id)

			if durability then
				durability = (durability / max_durability)
	
				if durability and durability >= 0 then
					local current_durability = math.floor(durability * 100)

					if AttritionDB.lowest then
						if not lowest or current_durability < lowest then
							lowest = current_durability
						end
					end

					local color      = crayon:GetThresholdHexColor(current_durability, 100)
					local _, _, cost = cost_tip:SetInventoryItem("player", id)
					total_price      = (total_price or 0) + cost
					cost             = cost > 0 and cash_to_string(cost).." |cffeeeeee-|r " or ""
					slots[slot]      = cost.."|cff"..color..current_durability.."%|r" 

					if not AttritionDB.lowest then
						average = (average or 0) + durability
						count   = (count   or 0) + 1
					end
				end
			end
		end
	end

	if (AttritionDB.lowest and not lowest) or (not AttritionDB.lowest and not average) then
		ldb_obj.text = ""

		return
	end

	if not AttritionDB.lowest then average = math.floor((average / count) * 100) end

	local color = crayon:GetThresholdHexColor(AttritionDB.lowest and lowest or average, 100)

	ldb_obj.text = "|cff"..color..(AttritionDB.lowest and lowest or average).."%|r"
end

function ldb_obj.OnTooltipShow(tip)
	if not ldb_tip then ldb_tip = tip end

	tip:ClearLines()
	tip:AddLine("Attrition: A DataBroker Plugin")
	tip:AddLine(" ")

	if ldb_obj.text == "" then
		tip:AddLine("|cff8888eeYou have no items with durability currently equipped.|r")

	else	tip:AddLine("|cff8888eeYour equipped items, their repair costs and durability:|r")
	
		for slot, stats in pairs(slots) do
			if type(stats) == "string" then
				local link  = GetInventoryItemLink("player", GetInventorySlotInfo(slot.."Slot"))
				tip:AddDoubleLine(link, stats)
			end
		end

		tip:AddLine(" ")
		tip:AddLine("|cffffd700Total Repair Cost:|r "..cash_to_string(total_price))
	end

	tip:AddLine(" ")
	tip:AddLine("|cffffd700Displaying:|r |cffeeeeee"..(AttritionDB.lowest and "Most damaged item" or "Average of all damaged items")..".|r")
	tip:AddLine("|cffffd700Auto-Repair:|r "..(AttritionDB.repair and "|cff88ff88Enabled|r" or "|cffff8888Disabled|r"))

	if AttritionDB.repair then
		tip:AddLine("|cffffd700Charging:|r |cffeeeeeeYour "..(AttritionDB.guild and "guild's" or "own").." funds.|r")

		if AttritionDB.guild then
			tip:AddLine(" ")
			tip:AddLine("|cffff6666Warning:|r |cffeeeeeeIf your guild's bank doesn't have the funds to cover the auto-repair,\nyour own money will be used instead.  Use this feature at your own risk!|r")
		end
	end

	tip:AddLine(" ")
	tip:AddLine("|cff69b950Click: |cffeeeeeeToggles between showing average of all items or most damaged.|r")
	tip:AddLine("|cff69b950Alt + Click:|r |cffeeeeeeToggles armour auto-repair feature on/off.|r")
	tip:AddLine("|cff69b950Shift + Click:|r |cffeeeeeeToggles on/off charging your guild's bank for repair costs.|r")

	tip:Show()
end

function ldb_obj.OnClick()
	if IsAltKeyDown() then
		AttritionDB.repair = not AttritionDB.repair

	elseif IsShiftKeyDown() then
		AttritionDB.guild = not AttritionDB.guild

	else	AttritionDB.lowest = not AttritionDB.lowest
		set_ldb_obj_text()
	end

	ldb_obj.OnTooltipShow(ldb_tip)
end

cost_tip:SetOwner(WorldFrame, "ANCHOR_NONE")

frame:RegisterEvent"ADDON_LOADED"
frame:RegisterEvent"MERCHANT_SHOW"
frame:RegisterEvent"MERCHANT_CLOSED"
frame:RegisterEvent"PLAYER_REGEN_ENABLED"
frame:RegisterEvent"PLAYER_DEAD"
frame:RegisterEvent"PLAYER_UNGHOST"
frame:RegisterEvent"UPDATE_INVENTORY_ALERTS"

frame:SetScript("OnEvent", set_ldb_obj_text)